<?php

/*
 * This file is part of the VITICORE2 package.
 *
 * (c) CP Creation <web@cpcreation.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace CpCreation\VitiTheme\Theme;

final readonly class ThemeResolver
{
    public function __construct(private ThemeRegistry $registry)
    {
    }

    public function active(): ThemeDefinition
    {
        return $this->registry->active();
    }
}
